package jdk.management.resource.internal.inst;

import java.net.SocketAddress;
import java.net.InetAddress;
import jdk.internal.instrumentation.InstrumentationMethod;
import jdk.management.resource.ResourceRequest;
import jdk.management.resource.ResourceRequestDeniedException;
import java.io.IOException;
import jdk.management.resource.internal.ResourceIdImpl;
import jdk.management.resource.internal.ApproverGroup;
import java.net.Socket;
import jdk.internal.instrumentation.InstrumentationTarget;

@InstrumentationTarget("java.net.ServerSocket")
final class ServerSocketRMHooks
{
    private Object closeLock;
    
    ServerSocketRMHooks() {
    }
    
    @InstrumentationMethod
    public Socket accept() throws IOException {
        long request = 0L;
        long n = 0L;
        Socket accept = null;
        ResourceIdImpl of = null;
        ResourceRequest approver = null;
        try {
            accept = this.accept();
            n = 1L;
            approver = ApproverGroup.SOCKET_OPEN_GROUP.getApprover(accept);
            of = ResourceIdImpl.of(accept.getLocalAddress());
            try {
                request = approver.request(1L, of);
                if (request < 1L) {
                    try {
                        accept.close();
                    }
                    catch (IOException ex) {}
                    throw new IOException("Resource limited: too many open sockets");
                }
            }
            catch (ResourceRequestDeniedException cause) {
                try {
                    accept.close();
                }
                catch (IOException ex2) {}
                throw new IOException("Resource limited: too many open sockets", cause);
            }
            n = 1L;
        }
        finally {
            if (approver != null) {
                approver.request(-(request - n), of);
            }
        }
        return accept;
    }
    
    @InstrumentationMethod
    public InetAddress getInetAddress() {
        return this.getInetAddress();
    }
    
    @InstrumentationMethod
    public void bind(SocketAddress socketAddress, int n) throws IOException {
        ResourceIdImpl of = null;
        ResourceRequest approver = null;
        long request = 0L;
        if (!this.isBound()) {
            of = ResourceIdImpl.of(socketAddress);
            approver = ApproverGroup.SOCKET_OPEN_GROUP.getApprover(this);
            request = approver.request(1L, of);
            if (request < 1L) {
                throw new ResourceRequestDeniedException("Resource limited: too many open sockets");
            }
        }
        int n2 = 0;
        try {
            this.bind(socketAddress, n);
            n2 = 1;
        }
        finally {
            if (approver != null) {
                approver.request(-(request - (long)n2), of);
            }
        }
    }
    
    @InstrumentationMethod
    public boolean isBound() {
        return this.isBound();
    }
    
    @InstrumentationMethod
    public boolean isClosed() {
        return this.isClosed();
    }
    
    @InstrumentationMethod
    public void close() throws IOException {
        synchronized (this.closeLock) {
            if (this.isClosed()) {
                return;
            }
        }
        int bound = this.isBound();
        InetAddress inetAddress = this.getInetAddress();
        try {
            this.close();
        }
        finally {
            if (!(bound == 0)) {
                ApproverGroup.SOCKET_OPEN_GROUP.getApprover(this).request(-1L, ResourceIdImpl.of(inetAddress));
            }
        }
    }
}
